@@ -1,5 +1,26 @@
 package SQL::Translator::Parser::DBI;
 
+# -------------------------------------------------------------------
+# $Id: DBI.pm,v 1.4 2003/10/10 15:52:07 kycl4rk Exp $
+# -------------------------------------------------------------------
+# Copyright (C) 2003 Ken Y. Clark <kclark@cpan.org>,
+#                    darren chamberlain <darren@cpan.org>
+#
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU General Public License as
+# published by the Free Software Foundation; version 2.
+#
+# This program is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+# 02111-1307  USA
+# -------------------------------------------------------------------
+
 =head1 NAME
 
 SQL::Translator::Parser::DBI - "parser" for DBI handles
@@ -9,12 +30,7 @@ SQL::Translator::Parser::DBI - "parser" for DBI handles
   use DBI;
   use SQL::Translator;
 
-  my $dbh = DBI->connect('dsn', 'user', 'pass',
-      {
-          RaiseError       => 1,
-          FetchHashKeyName => 'NAME_lc',
-      }
-  );
+  my $dbh = DBI->connect(...);
 
   my $translator  =  SQL::Translator->new(
       parser      => 'DBI',
@@ -25,19 +41,17 @@ Or:
 
   use SQL::Translator;
 
-  my $translator      =  SQL::Translator->new(
-      parser          => 'DBI',
-      parser_args     => {
-          dsn         => 'dbi:mysql:FOO',
-          db_user     => 'guest',
-          db_password => 'password',
-    }
+  my $translator  =  SQL::Translator->new(
+      parser      => 'DBI',
+      dsn         => 'dbi:mysql:FOO',
+      db_user     => 'guest',
+      db_password => 'password',
   );
 
 =head1 DESCRIPTION
 
-This parser accepts an open database handle (or the arguments to create
-one) and queries the database directly for the information.
+This parser accepts an open database handle (or the arguments to create 
+one) and queries the database directly for the information.  
 
 The following are acceptable arguments:
 
@@ -45,9 +59,7 @@ The following are acceptable arguments:
 
 =item * dbh
 
-An open DBI database handle.  NB:  Be sure to create the database with the
-"FetchHashKeyName => 'NAME_lc'" option as all the DBI parsers expect
-lowercased column names.
+An open DBI database handle.
 
 =item * dsn
 
@@ -67,7 +79,7 @@ There is no need to specify which type of database you are querying as
 this is determined automatically by inspecting $dbh->{'Driver'}{'Name'}.
 If a parser exists for your database, it will be used automatically;
 if not, the code will fail automatically (and you can write the parser
-and contribute it to the project!).
+and contribute it to the project!).  
 
 Currently parsers exist for the following databases:
 
@@ -96,27 +108,29 @@ query Oracle directly and skip the parsing of a text file, too.
 
 =cut
 
+# -------------------------------------------------------------------
+
 use strict;
-use warnings;
 use DBI;
-our @EXPORT;
-our $VERSION = '1.59';
+use vars qw($VERSION @EXPORT);
+$VERSION = sprintf "%d.%02d", q$Revision: 1.4 $ =~ /(\d+)\.(\d+)/;
 
 use constant DRIVERS => {
     mysql            => 'MySQL',
-    odbc             => 'SQLServer',
-    oracle           => 'Oracle',
-    pg               => 'PostgreSQL',
     sqlite           => 'SQLite',
     sybase           => 'Sybase',
     pg               => 'PostgreSQL',
-    db2              => 'DB2',
 };
 
 use Exporter;
 
 use SQL::Translator::Utils qw(debug);
 
+use SQL::Translator::Parser::DBI::MySQL;
+use SQL::Translator::Parser::DBI::SQLite;
+use SQL::Translator::Parser::DBI::Sybase;
+use SQL::Translator::Parser::DBI::PostgreSQL;
+
 use base qw(Exporter);
 @EXPORT = qw(parse);
 
@@ -134,13 +148,13 @@ sub parse {
 
     unless ( $dbh ) {
         die 'No DSN' unless $dsn;
-        $dbh = DBI->connect( $dsn, $db_user, $db_password,
+        $dbh = DBI->connect( $dsn, $db_user, $db_password, 
             {
                 FetchHashKeyName => 'NAME_lc',
                 LongReadLen      => 3000,
                 LongTruncOk      => 1,
                 RaiseError       => 1,
-            }
+            } 
         );
     }
 
@@ -151,7 +165,11 @@ sub parse {
     my $pkg     = "SQL::Translator::Parser::DBI::$driver";
     my $sub     = $pkg.'::parse';
 
-    SQL::Translator::load( $pkg );
+    #
+    # I can't get this to work.  I seem to have to have the "use"
+    # statements above.
+    #
+#    $tr->load( $pkg );
 
     eval {
         no strict 'refs';
@@ -167,6 +185,7 @@ sub parse {
 
 1;
 
+# -------------------------------------------------------------------
 =pod
 
 =head1 AUTHOR
