@@ -1,9 +1,10 @@
 package SQL::Translator::Parser::YAML;
 
 # -------------------------------------------------------------------
-# $Id: YAML.pm,v 1.8 2006/03/10 14:55:19 grommit Exp $
+# $Id: YAML.pm,v 1.3 2003/10/09 21:48:55 kycl4rk Exp $
 # -------------------------------------------------------------------
-# Copyright (C) 2002-4 SQLFairy Authors
+# Copyright (C) 2003 darren chamberlain <darren@cpan.org>,
+#   Ken Y. Clark <kclark@cpan.org>.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -22,7 +23,7 @@ package SQL::Translator::Parser::YAML;
 
 use strict;
 use vars qw($VERSION);
-$VERSION = sprintf "%d.%02d", q$Revision: 1.8 $ =~ /(\d+)\.(\d+)/;
+$VERSION = sprintf "%d.%02d", q$Revision: 1.3 $ =~ /(\d+)\.(\d+)/;
 
 use SQL::Translator::Schema;
 use SQL::Translator::Utils qw(header_comment);
@@ -34,7 +35,7 @@ sub parse {
     $data = Load($data);
     $data = $data->{'schema'};
 
-    warn "YAML data:",Dumper( $data ) if $translator->debug;
+    warn Dumper( $data ) if $translator->debug;
 
     my $schema = $translator->schema;
 
@@ -44,14 +45,13 @@ sub parse {
     my @tables = 
         map   { $data->{'tables'}{ $_->[1] } }
         sort  { $a->[0] <=> $b->[0] }
-        map   { [ $data->{'tables'}{ $_ }{'order'} || 0, $_ ] }
+        map   { [ $data->{'tables'}{ $_ }{'order'}, $_ ] }
         keys %{ $data->{'tables'} }
     ;
 
     for my $tdata ( @tables ) {
         my $table = $schema->add_table(
             name  => $tdata->{'name'},
-            extra => $tdata->{'extra'},
         ) or die $schema->error;
 
         my @fields = 
@@ -66,14 +66,6 @@ sub parse {
             $table->primary_key( $fdata->{'name'} ) 
                 if $fdata->{'is_primary_key'};
         }
-
-        for my $idata ( @{ $tdata->{'indices'} || [] } ) {
-            $table->add_index( %$idata ) or die $table->error;
-        }
-
-        for my $cdata ( @{ $tdata->{'constraints'} || [] } ) {
-            $table->add_constraint( %$cdata ) or die $table->error;
-        }
     }
 
     #
@@ -118,18 +110,6 @@ sub parse {
         $schema->add_procedure( %$tdata ) or die $schema->error;
     }
 
-    if ( my $tr_data = $data->{'translator'} ) {
-        $translator->add_drop_table( $tr_data->{'add_drop_table'} );
-        $translator->filename( $tr_data->{'filename'} );
-        $translator->no_comments( $tr_data->{'no_comments'} );
-        $translator->parser_args( $tr_data->{'parser_args'} );
-        $translator->producer_args( $tr_data->{'producer_args'} );
-        $translator->parser_type( $tr_data->{'parser_type'} );
-        $translator->producer_type( $tr_data->{'producer_type'} );
-        $translator->show_warnings( $tr_data->{'show_warnings'} );
-        $translator->trace( $tr_data->{'trace'} );
-    }
-
     return 1;
 }
 
@@ -151,7 +131,7 @@ SQL::Translator::Parser::YAML - Parse a YAML representation of a schema
 
 C<SQL::Translator::Parser::YAML> parses a schema serialized with YAML.
 
-=head1 AUTHORS
+=head1 AUTHOR
 
 Darren Chamberlain E<lt>darren@cpan.orgE<gt>,
 Ken Y. Clark E<lt>kclark@cpan.orgE<gt>.
